@@ -74,17 +74,18 @@ I_FIELDS = ["name", "admin_state", "os",
             "nic.ips", "nic.macs", "nic.modes", "nic.uuids", "nic.names",
             "nic.links", "nic.networks", "nic.networks.names", "nic.bridges",
             "network_port",
-            "disk.sizes", "disk_usage", "disk.uuids", "disk.names",
+            "disk.sizes", "disk.spindles", "disk_usage", "disk.uuids",
+            "disk.names",
             "beparams", "hvparams",
             "oper_state", "oper_ram", "oper_vcpus", "status",
             "custom_hvparams", "custom_beparams", "custom_nicparams",
             ] + _COMMON_FIELDS
 
 N_FIELDS = ["name", "offline", "master_candidate", "drained",
-            "dtotal", "dfree",
+            "dtotal", "dfree", "sptotal", "spfree",
             "mtotal", "mnode", "mfree",
             "pinst_cnt", "sinst_cnt",
-            "ctotal", "cnodes", "csockets",
+            "ctotal", "cnos", "cnodes", "csockets",
             "pip", "sip", "role",
             "pinst_list", "sinst_list",
             "master_capable", "vm_capable",
